@@ -22,8 +22,17 @@ from typing import (
 
 import numpy as np
 
+from ._nd2decode import _decode_custom_data, decode_metadata, unnest_experiments
 from ._util import AXIS, VoxelSize, get_reader, is_supported_file
-from .structures import ROI, Attributes, ExpLoop, FrameMetadata, Metadata, XYPosLoop
+from .structures import (
+    ROI,
+    Attributes,
+    ExpLoop,
+    FrameMetadata,
+    Metadata,
+    XYPosLoop,
+    ZStackLoop,
+)
 
 try:
     from functools import cached_property
@@ -38,6 +47,7 @@ if TYPE_CHECKING:
     import xarray as xr
     from typing_extensions import Literal
 
+    from ._binary import BinaryLayers
     from ._sdk.latest import ND2Reader as LatestSDKReader
     from .structures import Position
 
@@ -246,8 +256,6 @@ class ND2File:
                 "unstructured_metadata not available for legacy files"
             )
 
-        from ._nd2decode import decode_metadata, unnest_experiments
-
         output: Dict[str, Any] = {}
 
         rdr = cast("LatestSDKReader", self._rdr)
@@ -698,6 +706,123 @@ class ND2File:
             extra = ""
         return f"<ND2File at {hex(id(self))}{extra}>"
 
+    @cached_property
+    def recorded_data(self) -> Dict[str, Union[np.ndarray, Sequence]]:
+        """Return tabular data recorded for each frame of the experiment.
+
+        This method returns a dict of equal-length sequences (passable to
+        pd.DataFrame()). It matches the tabular data reported in the Image Properties >
+        Recorded Data tab of the NIS Viewer.
+
+        (There will be a column for each tag in the `CustomDataV2_0` section of
+        `ND2File.custom_data`)
+
+        Legacy ND2 files are not supported.
+        """
+        if self.is_legacy:
+            warnings.warn(
+                "`recorded_data` is not supported for legacy ND2 files", UserWarning
+            )
+            return {}
+        rdr = cast("LatestSDKReader", self._rdr)
+
+        cd = self.custom_data
+        if "CustomDataV2_0" not in cd:
+            return {}
+        try:
+            tags: dict = cd["CustomDataV2_0"]["CustomTagDescription_v1.0"]
+        except KeyError:
+            warnings.warn(
+                "Could not find 'CustomTagDescription_v1' tag, please open an issue "
+                "with this nd2 file at https://github.com/tlambert03/nd2/issues/new",
+            )
+            return {}
+
+        # tags will be a dict of dicts: eg:
+        # {'Tag0': {'ID': 'X', 'Type': 3, 'Group': 1, 'Size': 5000, 'Desc': 'X Coord', 'Unit': 'µm'}}  # noqa
+        # FIXME: technically, it is possible to have multiple tags with the same Desc
+        # (e.g. for IDs PFS_OFFSET and Z2). In the current implementation, the
+        # 2nd tag will overwrite the first one.
+        data: Dict[str, Union[np.ndarray, Sequence]] = {}
+        with contextlib.suppress(KeyError):
+            chunk = rdr._get_meta_chunk("CustomData|AcqTimesCache")
+            data["Time [s]"] = np.frombuffer(chunk) / 1000
+
+        try:
+            z_idx = [AXIS._MAP[c[1]] for c in self._rdr._coord_info()].index(AXIS.Z)
+        except ValueError:
+            pass
+        else:
+            # TODO: this is probably slow... and could definitely be improved
+            z_loop = cast("ZStackLoop", self.experiment[z_idx])
+            z_positions = [
+                z_loop.parameters.stepUm * (i - z_loop.parameters.homeIndex)
+                for i in range(z_loop.count)
+            ]
+            if not z_loop.parameters.bottomToTop:
+                z_positions = list(reversed(z_positions))
+
+            data["Z-Series"] = np.array(
+                [
+                    z_positions[rdr._coords_from_seq_index(i)[z_idx]]
+                    for i in range(self.attributes.sequenceCount)
+                ]
+            )
+
+        for tag in tags.values():
+            header = f"{tag['Desc']}"
+            if tag["Unit"]:
+                header += f" [{tag['Unit']}]"
+            raw = rdr._get_meta_chunk(f"CustomData|{tag['ID']}")
+            _type = tag["Type"]
+            if _type == 1:
+                warnings.warn(
+                    f"{header!r} column skipped: "
+                    "(parsing string data is not yet implemented)"
+                )
+            else:
+                data[header] = _decode_custom_data(raw, _type, tag["Size"])
+
+        return data
+
+    @cached_property
+    def binary_data(self) -> BinaryLayers | None:
+        """Return binary layers embedded in the file.
+
+        The returned `BinaryLayers` object is an immutable sequence of `BinaryLayer`
+        objects, one for each binary layer in the file.  Each `BinaryLayer` object in
+        the sequence has a `name` attribute, and a `data` attribute which is list of
+        numpy arrays (or `None` if there was no binary mask for that frame).  The length
+        of the list will be the same as the number of sequence frames in this file
+        (i.e. `self.attributes.sequenceCount`).
+
+        Both the `BinaryLayers` and individual `BinaryLayer` objects can be cast to a
+        numpy array with `np.asarray()`, or by using the `.asarray()` method
+
+        Returns
+        -------
+        BinaryLayers | None
+            The binary layers embedded in the file, or None if there are no binary
+            layers.
+
+        Examples
+        --------
+        >>> f = ND2File("path/to/file.nd2")
+        >>> f.binary_data
+        <BinaryLayers with 4 layers>
+        >>> f.binary_data[0]  # the first binary layer
+        BinaryLayer(name='attached Widefield green (green color)',
+        comp_name='Widefield Green', comp_order=2, color=65280, color_mode=0,
+        state=524288, file_tag='RleZipBinarySequence_1_v1', layer_id=2)
+        >>> f.binary_data[0].data  # list of arrays
+        >>> np.asarray(f.binary_data[0])  # just the first binary mask
+        >>> np.asarray(f.binary_data).shape  # cast all layers to array
+        (4, 3, 4, 5, 32, 32)
+        """
+        from ._binary import BinaryLayers
+
+        return BinaryLayers.from_nd2file(self)
+
 
 @overload
 def imread(
